/*
 * tryhost -- a simple type of "ping"
 * 
 * Copyright (c) 1991 by William LeFebvre
 *
 * This software is free and may be freely redistributed, but the author
 * requires that this entire comment area remain intact.
 *
 * Written by William LeFebvre specifically for "xrlogin".
 *
 * syntax: tryhost IP-number timeout
 *
 *	"IP-number" is the internet address to try, and is in DECIMAL!
 *		Decimal was chosen to make it quick to encode and decode.
 *	"timeout" is the number of seconds to try before giving up.
 */

#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

/*
 *  We do these as predefined constants because:  (1) we are after speed and
 *  (2) they won't be changing very often, if at all.
 */

#define  TCP            6
#define  UDP           17
#define  DISCARD_PORT   9

extern int errno;
extern char *sys_errlist[];
extern int sys_nerr;

timeout()

{
    fputs("Connection timed out\n", stderr);
    exit(1);
}

char *err_msg()

{
    if (errno >= 0 && errno < sys_nerr)
    {
	return(sys_errlist[errno]);
    }
    else
    {
	return(sys_errlist[0]);
    }
}

main(argc, argv)

int argc;
char *argv[];

{
    int s;
    int len;
    struct sockaddr_in addr, host;

    if (argc < 3)
    {
	fputs("invalid arguments", stderr);
	exit (1);
    }

    if ((s = socket(PF_INET, SOCK_STREAM, TCP)) < 0)
    {
	perror("socket");
	exit (1);
    }

    bzero(&host, sizeof(host));

    alarm(atoi(argv[2]));
    signal(SIGALRM, timeout);

    host.sin_family = AF_INET;
    host.sin_port = htons(DISCARD_PORT);
    host.sin_addr.s_addr = htonl(atoi(argv[1]));
    if (connect(s, &host, sizeof(host)) < 0)
    {
	if (errno != ECONNREFUSED)
	{
	    fprintf(stderr, "%s\n", err_msg());
	    exit(1);
	}
	/* else:  if the machine can refuse, then it must be alive! */
    }

    close(s);
    exit(0);
}
